/* ============================================================
 * File  : mythtubeconfig.cpp
 * Author: Alvaro Gonzalez Sola <cojonuo@gmail.com>
 * Date  : 2008-08-26
 * Description :
 *
 * MythTube configurator
 *
 * Copyright 2008 by Alvaro Gonzalez Sola
 *
 * This program is free software; you can redistribute it
 * and/or modify it under the terms of the GNU General
 * Public License as published bythe Free Software Foundation;
 * either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * ============================================================ */

#ifndef MYTHTUBECONFIG_CPP
#define MYTHTUBECONFIG_CPP

#include <mythtv/mythcontext.h>
#include <mythtv/mythdirs.h>

#include "mythtubeconfig.h"
#include "tubedbutil.h"

using namespace std;

MythTubeConfig::MythTubeConfig(MythScreenStack *parent, const char *name)
    : MythScreenType(parent, name)
{

}

MythTubeConfig::~MythTubeConfig(){

}

bool MythTubeConfig::Create(void)
{
    bool foundtheme = false;

    foundtheme = LoadWindowFromXML("tube-ui.xml", "config", this);

    if (!foundtheme)
        return false;

    m_categoriesList = dynamic_cast<MythUIButtonList *> (GetChild("categorieslist"));
    m_siteList = dynamic_cast<MythUIButtonList *> (GetChild("sitelist"));

    if (!m_siteList || !m_categoriesList)
    {
        VERBOSE(VB_IMPORTANT, "Theme is missing critical theme elements.");
        return false;
    }

    if (!BuildFocusList())
        VERBOSE(VB_IMPORTANT, "Failed to build a focuslist. Something is wrong");

    connect(m_categoriesList, SIGNAL(itemSelected( MythUIButtonListItem*)),
            this, SLOT( showChannels(MythUIButtonListItem*)));
    connect(m_siteList, SIGNAL(itemClicked(MythUIButtonListItem*)),
            this, SLOT(toggleItem(MythUIButtonListItem*)));

    SetFocusWidget(m_categoriesList);

    loadData();

    return true;

}

bool MythTubeConfig::loadData(){

    QString filename = GetShareDir()
                       + "mythtube/channels.xml";
    QFile xmlFile(filename);

    if (!xmlFile.exists() || !xmlFile.open(QIODevice::ReadOnly)) {
        VERBOSE(VB_IMPORTANT, "MythTubeConfig: Cannot open channels.xml");
        return false;
    }

    QString errorMsg;
    int errorLine = 0;
    int errorColumn = 0;

    QDomDocument domDoc;

    if (!domDoc.setContent(&xmlFile, false, &errorMsg, &errorLine, &errorColumn))
    {
        VERBOSE(VB_IMPORTANT, "MythTubeConfig: Error in reading content of channels.xml");
        VERBOSE(VB_IMPORTANT, QString("MythTubeConfig: Error, parsing %1\n"
                                      "at line: %2  column: %3 msg: %4")
                                      .arg(filename).arg(errorLine)
                                      .arg(errorColumn).arg(errorMsg));
        return false;
    }

    QDomNodeList catList =
        domDoc.elementsByTagName(QString::fromLatin1("category"));

    QDomNode catNode;
    QDomNode siteNode;
    categoryList = new CategoryList();
    categoryList->clear();
    for (int i = 0; i < catList.count(); i++) {
        catNode = catList.item(i);

        ChannelsCategory* cat = new ChannelsCategory();

        cat->name = catNode.toElement().attribute("name");
        categoryList->append(cat);

        QDomNodeList siteList = catNode.childNodes();

        for (int j = 0; j < siteList.count(); j++) {
            siteNode = siteList.item(j);

            ChannelItem *channel = new ChannelItem();
            channel->name =
                siteNode.namedItem(QString("title")).toElement().text();
            channel->url =
                siteNode.namedItem(QString("url")).toElement().text();
            channel->lang =
                siteNode.namedItem(QString("language")).toElement().text();

            cat->channels.append(channel);
        }
    }

    xmlFile.close();

    for (ChannelsCategory* cat = categoryList->first();
         cat; cat = categoryList->next() ) {
        MythUIButtonListItem* item =
            new MythUIButtonListItem(m_categoriesList, cat->name);
        item->setData(cat);
    }

    showChannels(m_categoriesList->GetItemFirst());

    return true;
}

void MythTubeConfig::showChannels(MythUIButtonListItem* selected){

    if(selected){
        m_siteList->Reset();

        ChannelsCategory* cat = (ChannelsCategory*) selected->getData();
        if (cat) {

            for (ChannelItem * site = cat->channels.first();
                    site; site = cat->channels.next() ) {

                QString name = "(" + site->lang + ") " + site->name;

                MythUIButtonListItem* item =
                    new MythUIButtonListItem(m_siteList, name, 0, true,
                                        findInDB(site->name) ?
                                        MythUIButtonListItem::FullChecked :
                                        MythUIButtonListItem::NotChecked);

                item->setData(site);
            }
        }
    }
}

void MythTubeConfig::toggleItem(MythUIButtonListItem *item)
{
    if (!item || !item->getData())
        return;

    ChannelItem* site = (ChannelItem*) item->getData();

    bool checked = (item->state() == MythUIButtonListItem::FullChecked);

    if (!checked) {
        if (insertInDB(site->name, site->url, Channel::Feed))
        {
            item->setChecked(MythUIButtonListItem::FullChecked);
        }
    }
    else {
        if (removeFromDB(site->name))
        {
            item->setChecked(MythUIButtonListItem::NotChecked);
        }
    }
}

bool MythTubeConfig::keyPressEvent(QKeyEvent *event)
{
    if (GetFocusWidget()->keyPressEvent(event))
        return true;

    bool handled = false;
    QStringList actions;
    gContext->GetMainWindow()->TranslateKeyPress("Tube", event, actions);

    if (!handled && MythScreenType::keyPressEvent(event))
        handled = true;

    return handled;
}

#endif